//  This file is part of foo_timedisplay.
//
//  foo_timedisplay is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  foo_timedisplay is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with foo_timedisplay. If not, see <http://www.gnu.org/licenses/>.

#include "foo_timedisplay_callback.h"
#include "foo_timedisplay.h"

extern HWND g_hWnd;
extern HANDLE g_hUpdaterThreadPauseEvent;
extern int g_nHour;
extern int g_nMinute;
extern int g_nSecond;
extern int g_nMsec;
extern cfg_bool cfg_dispremainingtime;
extern foo_timedisplay* g_foo_timedisplay;

extern double g_dTrackLength;
extern double g_dCurrTrackPos;
extern double g_dRemainingPlaylistLength;
static bool g_fMsecTimerEnabled;
static UINT g_nMsecTimerID;
static int g_nStepsToWait;
static int g_nStepsToCorrect;
foo_timedisplay_callback g_foo_timedisplay_callback;
#ifdef DEBUG
static double g_fAccuracy = 0;
static int g_nAccuracyCounter = 1;
#endif
extern cfg_int cfg_flash;
extern HANDLE g_hResetFlashing;
extern cfg_bool cfg_customdisplayformat;
DWORD g_dwTrackStartTime;
extern HANDLE g_hScrollAccelerationOver;
extern HANDLE g_hScrollAccelerationInit;
extern cfg_bool cfg_scrollwhenpaused;
extern bool g_fScrollEnabled;
extern cfg_int cfg_scrollinterval;

foo_timedisplay_callback::foo_timedisplay_callback()
{
	g_nStepsToCorrect = 0;
	g_nStepsToWait = 0;
	g_nMsecTimerID = 0;
	g_dwTrackStartTime = 0;
}

foo_timedisplay_callback::~foo_timedisplay_callback()
{
	timeKillEvent( g_nMsecTimerID );
}

void CALLBACK foo_timedisplay_callback::msecTimerProc(UINT uTimerID, UINT uMsg, DWORD_PTR dwUser, DWORD_PTR dw1, DWORD_PTR dw2)
{
	if( g_nStepsToWait )
		g_nStepsToWait--;
	else
		g_nMsec++;
}

void foo_timedisplay_callback::CalculateTime( double p_time )
{
	div_t divres;

	divres = div( (int)p_time, 60 );
	g_nMinute = divres.quot;
	g_nSecond = divres.rem;
	divres = div( g_nMinute, 60 );
	g_nHour = divres.quot;
	g_nMinute = divres.rem;
}

void foo_timedisplay_callback::EnableMsecTimer()
{
	g_fMsecTimerEnabled = true;
}

void foo_timedisplay_callback::DisableMsecTimer()
{
	g_fMsecTimerEnabled = false;
	timeKillEvent( g_nMsecTimerID );
	g_nMsecTimerID = 0;
}

void foo_timedisplay_callback::on_playback_time( double p_time )
{
	static_api_ptr_t<playback_control> pPlaybackControl;

	if( g_nMsecTimerID == 0 && g_fMsecTimerEnabled && !pPlaybackControl->is_paused() )
		g_nMsecTimerID = timeSetEvent( 1, 0, msecTimerProc, 0, TIME_PERIODIC );

	CalculateTime( p_time );
	g_dTrackLength = pPlaybackControl->playback_get_length();
	g_dCurrTrackPos = p_time;

	if( g_nMsec > 1000 )
		g_nStepsToCorrect++;
	if( g_nMsec < 1000 )
		g_nStepsToCorrect--;
	if( g_nStepsToCorrect < 0 )
		g_nStepsToCorrect = 0;
#ifdef DEBUG
	g_fAccuracy += abs(1000-g_nMsec);
	g_fAccuracy = ((g_fAccuracy/2)/1000.0)*100.0;
	console::formatter() << "foo_timedisplay accuracy: " << 100-g_fAccuracy << "% - number of measures: " << g_nAccuracyCounter++;
#endif
	g_nMsec = 0;
	g_nStepsToWait = g_nStepsToCorrect;

	g_foo_timedisplay->ProcessDisplayFormatString( pPlaybackControl );
}

void foo_timedisplay_callback::on_playback_seek( double p_time )
{
	on_playback_time( p_time );
	SetEvent( g_hResetFlashing );
	if( IsWindowVisible( g_hWnd ) )
	{
		g_foo_timedisplay->UpdateDisplayString();
		RedrawWindow( g_hWnd, 0, 0, RDW_INVALIDATE );
	}
}

void foo_timedisplay_callback::on_playback_stop( play_control::t_stop_reason p_reason )
{
	on_playback_time( 0 );

	g_dRemainingPlaylistLength = 0;
	timeKillEvent( g_nMsecTimerID );
	g_nMsecTimerID = 0;

	SetEvent( g_hResetFlashing );
	ResetEvent( g_hUpdaterThreadPauseEvent );

	SetEvent( g_hScrollAccelerationInit );
	ResetEvent( g_hScrollAccelerationOver );
	g_foo_timedisplay->ResetScrollPos();

	if( IsWindowVisible( g_hWnd ) )
	{
		g_foo_timedisplay->UpdateDisplayString();
		RedrawWindow( g_hWnd, 0, 0, RDW_INVALIDATE );
	}
}

void foo_timedisplay_callback::on_playback_new_track( metadb_handle_ptr p_track )
{
	on_playback_time( 0 );

	SetEvent( g_hResetFlashing );
	SetEvent( g_hScrollAccelerationInit );
	ResetEvent( g_hScrollAccelerationOver );
	g_foo_timedisplay->ResetScrollPos();
	g_dwTrackStartTime = GetTickCount();

	if( IsWindowVisible( g_hWnd ) )
	{
		static_api_ptr_t<playback_control> pPlaybackControl;
		if( !pPlaybackControl->is_paused() )
		{
			if( !g_nMsecTimerID && g_fMsecTimerEnabled )
				g_nMsecTimerID = timeSetEvent( 1, 0, msecTimerProc, 0, TIME_PERIODIC );
			SetEvent( g_hUpdaterThreadPauseEvent );
		}

		g_foo_timedisplay->UpdateDisplayString();
		RedrawWindow( g_hWnd, 0, 0, RDW_INVALIDATE );
	}
}

void foo_timedisplay_callback::on_playback_pause( bool p_state )
{
	SetEvent( g_hResetFlashing );
	if( p_state ) // paused
	{
		timeKillEvent( g_nMsecTimerID );
		g_nMsecTimerID = 0;

		if( !( cfg_scrollwhenpaused && g_fScrollEnabled && cfg_scrollinterval > 0 ) )
			ResetEvent( g_hUpdaterThreadPauseEvent );
		RedrawWindow( g_hWnd, 0, 0, RDW_INVALIDATE );
	}
	else // unpaused
	{
		if( IsWindowVisible( g_hWnd ) )
		{
			if( !g_nMsecTimerID && g_fMsecTimerEnabled )
				g_nMsecTimerID = timeSetEvent( 1, 0, msecTimerProc, 0, TIME_PERIODIC );
			SetEvent( g_hUpdaterThreadPauseEvent );
		}
	}
}

void foo_timedisplay_callback::on_playback_dynamic_info( const file_info & p_info )
{
	if( !IsWindowVisible( g_hWnd ) )
		return;
	if( cfg_customdisplayformat )
	{
		static_api_ptr_t<playback_control> pPlaybackControl;
		g_foo_timedisplay->ProcessDisplayFormatString( pPlaybackControl );
		g_foo_timedisplay->UpdateDisplayString();
		RedrawWindow( g_hWnd, 0, 0, RDW_INVALIDATE );
	}
}
