//  This file is part of foo_timedisplay.
//
//  foo_timedisplay is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  foo_timedisplay is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with foo_timedisplay. If not, see <http://www.gnu.org/licenses/>.

#include "foo_timedisplay.h"
#include "foo_timedisplay_mainmenu_commands.h"
#include "guids.h"

extern cfg_bool cfg_timedisplay_enabled;
extern HWND g_hWnd;

static mainmenu_commands_factory_t<foo_timedisplay_mainmenu_commands> g_mainmenu_commands;

void foo_timedisplay_mainmenu_commands::execute( t_uint32 p_index, service_ptr_t<service_base> p_callback )
{
	if( p_index == 0 && core_api::assert_main_thread() ) {
		if( !cfg_timedisplay_enabled )
		{
			ShowWindow( g_hWnd, SW_SHOW );
			cfg_timedisplay_enabled = true;
		}
		else
		{
			ShowWindow( g_hWnd, SW_HIDE );
			cfg_timedisplay_enabled = false;
		}
	}
}

bool foo_timedisplay_mainmenu_commands::is_checked( t_uint32 p_index )
{
	if( p_index == 0 )
		return cfg_timedisplay_enabled;
	return false;
}

bool foo_timedisplay_mainmenu_commands::get_display( t_uint32 p_index, pfc::string_base &p_text, t_uint32 &p_flags )
{
	p_flags = 0;
	if( is_checked( p_index ) )
		p_flags |= flag_checked;
	get_name( p_index, p_text );
	return true;
}

bool foo_timedisplay_mainmenu_commands::get_description( t_uint32 p_index, pfc::string_base &p_out )
{
	if ( p_index == 0 )
		p_out = "Activates time display window.";
	else
		return false;
	return true;
}

GUID foo_timedisplay_mainmenu_commands::get_command( t_uint32 p_index )
{
	if( p_index == 0 )
		return guid_mainmenu_command;
	return pfc::guid_null;
}

void foo_timedisplay_mainmenu_commands::get_name( t_uint32 p_index, pfc::string_base &p_out )
{
	if( p_index == 0 )
		p_out = PLUGINNAME;
}
